# -*- coding: utf-8 -*-
"""
Created on Wed Oct  2 21:21:05 2019

@author: qchat
"""

            
# XPS Python class
#
#  for XPS-C8 Firmware V2.1.4
#
#  See Programmer's manual for more information on XPS function calls
import socket





class XPS:
    # Defines
    MAX_NB_SOCKETS = 100

    # Global variables
    __sockets = {}
    __usedSockets = {}
    __nbSockets = 0

    # Initialization Function
    def __init__ (self):
        XPS.__nbSockets = 0
        for socketId in range(self.MAX_NB_SOCKETS):
            XPS.__usedSockets[socketId] = 0

    # Send command and get return
    def __sendAndReceive (self, socketId, command):
        try:
            XPS.__sockets[socketId].send(command.encode())
            ret = XPS.__sockets[socketId].recv(1024).decode()
            while (ret.find(',EndOfAPI') == -1):
                ret += XPS.__sockets[socketId].recv(1024).decode()
        except socket.timeout:
            return [-2, '']
        except socket.error(errNb, errString):
            print('Socket error : ' + errString)
            return [-2, '']
        for i in range(len(ret)):
            if (ret[i] == ','):
                return [int(ret[0:i]), ret[i+1:-9]]

    # TCP_ConnectToServer
    def TCP_ConnectToServer (self, IP, port, timeOut):
        socketId = 0
        if (XPS.__nbSockets < self.MAX_NB_SOCKETS):
            while (XPS.__usedSockets[socketId] == 1 and socketId < self.MAX_NB_SOCKETS):
                socketId += 1
            if (socketId == self.MAX_NB_SOCKETS):
                return -1
        else:
            return -1

        XPS.__usedSockets[socketId] = 1
        XPS.__nbSockets += 1
        try:
            XPS.__sockets[socketId] = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            XPS.__sockets[socketId].connect((IP, int(port)))
            XPS.__sockets[socketId].settimeout(timeOut)
            XPS.__sockets[socketId].setblocking(1)
        except socket.error:
            return -1

        return socketId

    # TCP_SetTimeout
    def TCP_SetTimeout (self, socketId, timeOut):
        if (XPS.__usedSockets[socketId] == 1):
            XPS.__sockets[socketId].settimeout(timeOut)

    # TCP_CloseSocket
    def TCP_CloseSocket (self, socketId):
        if (socketId >= 0 and socketId < self.MAX_NB_SOCKETS):
            try:
                XPS.__sockets[socketId].close()
                XPS.__usedSockets[socketId] = 0
                XPS.__nbSockets -= 1
            except socket.error:
                pass

    # GetLibraryVersion
    def GetLibraryVersion (self):
        return ['XPS-C8 Firmware V2.1.4']

    # ElapsedTimeGet :  Return elapsed time from controller power on
    def ElapsedTimeGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'ElapsedTimeGet(double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # ErrorStringGet :  Return the error string corresponding to the error code
    def ErrorStringGet (self, socketId, ErrorCode):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'ErrorStringGet(' + str(ErrorCode) + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # FirmwareVersionGet :  Return firmware version
    def FirmwareVersionGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'FirmwareVersionGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # TCLScriptExecute :  Execute a TCL script from a TCL file
    def TCLScriptExecute (self, socketId, TCLFileName, TaskName, ParametersList):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'TCLScriptExecute(' + TCLFileName + ',' + TaskName + ',' + ParametersList + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # TCLScriptExecuteAndWait :  Execute a TCL script from a TCL file and wait the end of execution to return
    def TCLScriptExecuteAndWait (self, socketId, TCLFileName, TaskName, InputParametersList):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'TCLScriptExecuteAndWait(' + TCLFileName + ',' + TaskName + ',' + InputParametersList + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # TCLScriptKill :  Kill TCL Task
    def TCLScriptKill (self, socketId, TaskName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'TCLScriptKill(' + TaskName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # TimerGet :  Get a timer
    def TimerGet (self, socketId, TimerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'TimerGet(' + TimerName + ',int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # TimerSet :  Set a timer
    def TimerSet (self, socketId, TimerName, FrequencyTicks):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'TimerSet(' + TimerName + ',' + str(FrequencyTicks) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # Reboot :  Reboot the controller
    def Reboot (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'Reboot()'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # Login :  Log in
    def Login (self, socketId, Name, Password):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'Login(' + Name + ',' + Password + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # CloseAllOtherSockets :  Close all socket beside the one used to send this command
    def CloseAllOtherSockets (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'CloseAllOtherSockets()'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventAdd :  ** OBSOLETE ** Add an event
    def EventAdd (self, socketId, PositionerName, EventName, EventParameter, ActionName, ActionParameter1, ActionParameter2, ActionParameter3):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventAdd(' + PositionerName + ',' + EventName + ',' + EventParameter + ',' + ActionName + ',' + ActionParameter1 + ',' + ActionParameter2 + ',' + ActionParameter3 + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventGet :  ** OBSOLETE ** Read events and actions list
    def EventGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventGet(' + PositionerName + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventRemove :  ** OBSOLETE ** Delete an event
    def EventRemove (self, socketId, PositionerName, EventName, EventParameter):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventRemove(' + PositionerName + ',' + EventName + ',' + EventParameter + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventWait :  ** OBSOLETE ** Wait an event
    def EventWait (self, socketId, PositionerName, EventName, EventParameter):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventWait(' + PositionerName + ',' + EventName + ',' + EventParameter + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventExtendedConfigurationTriggerSet :  Configure one or several events
    def EventExtendedConfigurationTriggerSet (self, socketId, ExtendedEventName, EventParameter1, EventParameter2, EventParameter3, EventParameter4):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventExtendedConfigurationTriggerSet('
        for i in range(len(ExtendedEventName)):
            if (i > 0):
                command += ','
            command += ExtendedEventName[i] + ',' + EventParameter1[i] + ',' + EventParameter2[i] + ',' + EventParameter3[i] + ',' + EventParameter4[i]
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventExtendedConfigurationTriggerGet :  Read the event configuration
    def EventExtendedConfigurationTriggerGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventExtendedConfigurationTriggerGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventExtendedConfigurationActionSet :  Configure one or several actions
    def EventExtendedConfigurationActionSet (self, socketId, ExtendedActionName, ActionParameter1, ActionParameter2, ActionParameter3, ActionParameter4):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventExtendedConfigurationActionSet('
        for i in range(len(ExtendedActionName)):
            if (i > 0):
                command += ','
            command += ExtendedActionName[i] + ',' + ActionParameter1[i] + ',' + ActionParameter2[i] + ',' + ActionParameter3[i] + ',' + ActionParameter4[i]
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventExtendedConfigurationActionGet :  Read the action configuration
    def EventExtendedConfigurationActionGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventExtendedConfigurationActionGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventExtendedStart :  Launch the last event and action configuration and return an ID
    def EventExtendedStart (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventExtendedStart(int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # EventExtendedAllGet :  Read all event and action configurations
    def EventExtendedAllGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventExtendedAllGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventExtendedGet :  Read the event and action configuration defined by ID
    def EventExtendedGet (self, socketId, ID):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventExtendedGet(' + str(ID) + ',char *,char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventExtendedRemove :  Remove the event and action configuration defined by ID
    def EventExtendedRemove (self, socketId, ID):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventExtendedRemove(' + str(ID) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventExtendedWait :  Wait events from the last event configuration
    def EventExtendedWait (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventExtendedWait()'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringConfigurationGet : Read different mnemonique type
    def GatheringConfigurationGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringConfigurationGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringConfigurationSet :  Configuration acquisition
    def GatheringConfigurationSet (self, socketId, Type):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringConfigurationSet('
        for i in range(len(Type)):
            if (i > 0):
                command += ','
            command += Type[i]
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringCurrentNumberGet :  Maximum number of samples and current number during acquisition
    def GatheringCurrentNumberGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringCurrentNumberGet(int *,int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GatheringStopAndSave :  Stop acquisition and save data
    def GatheringStopAndSave (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringStopAndSave()'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringDataAcquire :  Acquire a configured data
    def GatheringDataAcquire (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringDataAcquire()'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringDataGet :  Get a data line from gathering buffer
    def GatheringDataGet (self, socketId, IndexPoint):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringDataGet(' + str(IndexPoint) + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringReset :  Empty the gathered data in memory to start new gathering from scratch
    def GatheringReset (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringReset()'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringExternalConfigurationSet :  Configuration acquisition
    def GatheringExternalConfigurationSet (self, socketId, Type):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringExternalConfigurationSet('
        for i in range(len(Type)):
            if (i > 0):
                command += ','
            command += Type[i]
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringExternalConfigurationGet :  Read different mnemonique type
    def GatheringExternalConfigurationGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringExternalConfigurationGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringExternalCurrentNumberGet :  Maximum number of samples and current number during acquisition
    def GatheringExternalCurrentNumberGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringExternalCurrentNumberGet(int *,int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GatheringExternalStopAndSave :  Stop acquisition and save data
    def GatheringExternalStopAndSave (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringExternalStopAndSave()'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GlobalArrayGet :  Get global array value
    def GlobalArrayGet (self, socketId, Number):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GlobalArrayGet(' + str(Number) + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GlobalArraySet :  Set global array value
    def GlobalArraySet (self, socketId, Number, ValueString):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GlobalArraySet(' + str(Number) + ',' + ValueString + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GPIOAnalogGet :  Read analog input or analog output for one or few input
    def GPIOAnalogGet (self, socketId, GPIOName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GPIOAnalogGet('
        for i in range(len(GPIOName)):
            if (i > 0):
                command += ','
            command += GPIOName[i] + ',' + 'double *'
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(len(GPIOName)):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GPIOAnalogSet :  Set analog output for one or few output
    def GPIOAnalogSet (self, socketId, GPIOName, AnalogOutputValue):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GPIOAnalogSet('
        for i in range(len(GPIOName)):
            if (i > 0):
                command += ','
            command += GPIOName[i] + ',' + str(AnalogOutputValue[i])
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GPIOAnalogGainGet :  Read analog input gain (1, 2, 4 or 8) for one or few input
    def GPIOAnalogGainGet (self, socketId, GPIOName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GPIOAnalogGainGet('
        for i in range(len(GPIOName)):
            if (i > 0):
                command += ','
            command += GPIOName[i] + ',' + 'int *'
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(len(GPIOName)):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GPIOAnalogGainSet :  Set analog input gain (1, 2, 4 or 8) for one or few input
    def GPIOAnalogGainSet (self, socketId, GPIOName, AnalogInputGainValue):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GPIOAnalogGainSet('
        for i in range(len(GPIOName)):
            if (i > 0):
                command += ','
            command += GPIOName[i] + ',' + str(AnalogInputGainValue[i])
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GPIODigitalGet :  Read digital output or digital input
    def GPIODigitalGet (self, socketId, GPIOName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GPIODigitalGet(' + GPIOName + ',unsigned short *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # GPIODigitalSet :  Set Digital Output for one or few output TTL
    def GPIODigitalSet (self, socketId, GPIOName, Mask, DigitalOutputValue):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GPIODigitalSet(' + GPIOName + ',' + str(Mask) + ',' + str(DigitalOutputValue) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupAnalogTrackingModeEnable :  Enable Analog Tracking mode on selected group
    def GroupAnalogTrackingModeEnable (self, socketId, GroupName, Type):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupAnalogTrackingModeEnable(' + GroupName + ',' + Type + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupAnalogTrackingModeDisable :  Disable Analog Tracking mode on selected group
    def GroupAnalogTrackingModeDisable (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupAnalogTrackingModeDisable(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupCorrectorOutputGet :  Return corrector outputs
    def GroupCorrectorOutputGet (self, socketId, GroupName, nbElement):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupCorrectorOutputGet(' + GroupName + ','
        for i in range(nbElement):
            if (i > 0):
                command += ','
            command += 'double *'
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(nbElement):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GroupHomeSearch :  Start home search sequence
    def GroupHomeSearch (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupHomeSearch(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupHomeSearchAndRelativeMove :  Start home search sequence and execute a displacement
    def GroupHomeSearchAndRelativeMove (self, socketId, GroupName, TargetDisplacement):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupHomeSearchAndRelativeMove(' + GroupName + ','
        for i in range(len(TargetDisplacement)):
            if (i > 0):
                command += ','
            command += str(TargetDisplacement[i])
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupInitialize :  Start the initialization
    def GroupInitialize (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupInitialize(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupInitializeWithEncoderCalibration :  Start the initialization with encoder calibration
    def GroupInitializeWithEncoderCalibration (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupInitializeWithEncoderCalibration(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupJogParametersSet :  Modify Jog parameters on selected group and activate the continuous move
    def GroupJogParametersSet (self, socketId, GroupName, Velocity, Acceleration):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupJogParametersSet(' + GroupName + ','
        for i in range(len(Velocity)):
            if (i > 0):
                command += ','
            command += str(Velocity[i]) + ',' + str(Acceleration[i])
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupJogParametersGet :  Get Jog parameters on selected group
    def GroupJogParametersGet (self, socketId, GroupName, nbElement):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupJogParametersGet(' + GroupName + ','
        for i in range(nbElement):
            if (i > 0):
                command += ','
            command += 'double *' + ',' + 'double *'
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(nbElement*2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GroupJogCurrentGet :  Get Jog current on selected group
    def GroupJogCurrentGet (self, socketId, GroupName, nbElement):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupJogCurrentGet(' + GroupName + ','
        for i in range(nbElement):
            if (i > 0):
                command += ','
            command += 'double *' + ',' + 'double *'
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(nbElement*2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GroupJogModeEnable :  Enable Jog mode on selected group
    def GroupJogModeEnable (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupJogModeEnable(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupJogModeDisable :  Disable Jog mode on selected group
    def GroupJogModeDisable (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupJogModeDisable(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupKill :  Kill the group
    def GroupKill (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupKill(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupMoveAbort :  Abort a move
    def GroupMoveAbort (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupMoveAbort(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupMoveAbsolute :  Do an absolute move
    def GroupMoveAbsolute (self, socketId, GroupName, TargetPosition):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupMoveAbsolute(' + GroupName + ','
        for i in range(len(TargetPosition)):
            if (i > 0):
                command += ','
            command += str(TargetPosition[i])
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupMoveRelative :  Do a relative move
    def GroupMoveRelative (self, socketId, GroupName, TargetDisplacement):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupMoveRelative(' + GroupName + ','
        for i in range(len(TargetDisplacement)):
            if (i > 0):
                command += ','
            command += str(TargetDisplacement[i])
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupMotionDisable :  Set Motion disable on selected group
    def GroupMotionDisable (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupMotionDisable(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupMotionEnable :  Set Motion enable on selected group
    def GroupMotionEnable (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupMotionEnable(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupPositionCurrentGet :  Return current positions
    def GroupPositionCurrentGet (self, socketId, GroupName, nbElement):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupPositionCurrentGet(' + GroupName + ','
        for i in range(nbElement):
            if (i > 0):
                command += ','
            command += 'double *'
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(nbElement):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GroupPositionSetpointGet :  Return setpoint positions
    def GroupPositionSetpointGet (self, socketId, GroupName, nbElement):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupPositionSetpointGet(' + GroupName + ','
        for i in range(nbElement):
            if (i > 0):
                command += ','
            command += 'double *'
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(nbElement):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GroupPositionTargetGet :  Return target positions
    def GroupPositionTargetGet (self, socketId, GroupName, nbElement):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupPositionTargetGet(' + GroupName + ','
        for i in range(nbElement):
            if (i > 0):
                command += ','
            command += 'double *'
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(nbElement):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GroupReferencingActionExecute :  Execute an action in referencing mode
    def GroupReferencingActionExecute (self, socketId, PositionerName, ReferencingAction, ReferencingSensor, ReferencingParameter):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupReferencingActionExecute(' + PositionerName + ',' + ReferencingAction + ',' + ReferencingSensor + ',' + str(ReferencingParameter) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupReferencingStart :  Enter referencing mode
    def GroupReferencingStart (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupReferencingStart(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupReferencingStop :  Exit referencing mode
    def GroupReferencingStop (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupReferencingStop(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupStatusGet :  Return group status
    def GroupStatusGet (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupStatusGet(' + GroupName + ',int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # GroupStatusStringGet :  Return the group status string corresponding to the group status code
    def GroupStatusStringGet (self, socketId, GroupStatusCode):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupStatusStringGet(' + str(GroupStatusCode) + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupVelocityCurrentGet :  Return current velocities
    def GroupVelocityCurrentGet (self, socketId, GroupName, nbElement):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupVelocityCurrentGet(' + GroupName + ','
        for i in range(nbElement):
            if (i > 0):
                command += ','
            command += 'double *'
        command += ')'

        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(nbElement):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # KillAll :  Put all groups in 'Not initialized' state
    def KillAll (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'KillAll()'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerAnalogTrackingPositionParametersGet :  Read dynamic parameters for one axe of a group for a future analog tracking position
    def PositionerAnalogTrackingPositionParametersGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerAnalogTrackingPositionParametersGet(' + PositionerName + ',char *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(4):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerAnalogTrackingPositionParametersSet :  Update dynamic parameters for one axe of a group for a future analog tracking position
    def PositionerAnalogTrackingPositionParametersSet (self, socketId, PositionerName, GPIOName, Offset, Scale, Velocity, Acceleration):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerAnalogTrackingPositionParametersSet(' + PositionerName + ',' + GPIOName + ',' + str(Offset) + ',' + str(Scale) + ',' + str(Velocity) + ',' + str(Acceleration) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerAnalogTrackingVelocityParametersGet :  Read dynamic parameters for one axe of a group for a future analog tracking velocity
    def PositionerAnalogTrackingVelocityParametersGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerAnalogTrackingVelocityParametersGet(' + PositionerName + ',char *,double *,double *,double *,int *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(6):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerAnalogTrackingVelocityParametersSet :  Update dynamic parameters for one axe of a group for a future analog tracking velocity
    def PositionerAnalogTrackingVelocityParametersSet (self, socketId, PositionerName, GPIOName, Offset, Scale, DeadBandThreshold, Order, Velocity, Acceleration):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerAnalogTrackingVelocityParametersSet(' + PositionerName + ',' + GPIOName + ',' + str(Offset) + ',' + str(Scale) + ',' + str(DeadBandThreshold) + ',' + str(Order) + ',' + str(Velocity) + ',' + str(Acceleration) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerBacklashGet :  Read backlash value and status
    def PositionerBacklashGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerBacklashGet(' + PositionerName + ',double *,char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerBacklashSet :  Set backlash value
    def PositionerBacklashSet (self, socketId, PositionerName, BacklashValue):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerBacklashSet(' + PositionerName + ',' + str(BacklashValue) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerBacklashEnable :  Enable the backlash
    def PositionerBacklashEnable (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerBacklashEnable(' + PositionerName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerBacklashDisable :  Disable the backlash
    def PositionerBacklashDisable (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerBacklashDisable(' + PositionerName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerCorrectorNotchFiltersSet :  Update filters parameters
    def PositionerCorrectorNotchFiltersSet (self, socketId, PositionerName, NotchFrequency1, NotchBandwith1, NotchGain1, NotchFrequency2, NotchBandwith2, NotchGain2):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorNotchFiltersSet(' + PositionerName + ',' + str(NotchFrequency1) + ',' + str(NotchBandwith1) + ',' + str(NotchGain1) + ',' + str(NotchFrequency2) + ',' + str(NotchBandwith2) + ',' + str(NotchGain2) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerCorrectorNotchFiltersGet :  Read filters parameters
    def PositionerCorrectorNotchFiltersGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorNotchFiltersGet(' + PositionerName + ',double *,double *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(6):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerCorrectorPIDFFAccelerationSet :  Update corrector parameters
    def PositionerCorrectorPIDFFAccelerationSet (self, socketId, PositionerName, ClosedLoopStatus, KP, KI, KD, KS, IntegrationTime, DerivativeFilterCutOffFrequency, GKP, GKI, GKD, KForm, FeedForwardGainAcceleration):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorPIDFFAccelerationSet(' + PositionerName + ',' + str(ClosedLoopStatus) + ',' + str(KP) + ',' + str(KI) + ',' + str(KD) + ',' + str(KS) + ',' + str(IntegrationTime) + ',' + str(DerivativeFilterCutOffFrequency) + ',' + str(GKP) + ',' + str(GKI) + ',' + str(GKD) + ',' + str(KForm) + ',' + str(FeedForwardGainAcceleration) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerCorrectorPIDFFAccelerationGet :  Read corrector parameters
    def PositionerCorrectorPIDFFAccelerationGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorPIDFFAccelerationGet(' + PositionerName + ',bool *,double *,double *,double *,double *,double *,double *,double *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(12):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerCorrectorPIDFFVelocitySet :  Update corrector parameters
    def PositionerCorrectorPIDFFVelocitySet (self, socketId, PositionerName, ClosedLoopStatus, KP, KI, KD, KS, IntegrationTime, DerivativeFilterCutOffFrequency, GKP, GKI, GKD, KForm, FeedForwardGainVelocity):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorPIDFFVelocitySet(' + PositionerName + ',' + str(ClosedLoopStatus) + ',' + str(KP) + ',' + str(KI) + ',' + str(KD) + ',' + str(KS) + ',' + str(IntegrationTime) + ',' + str(DerivativeFilterCutOffFrequency) + ',' + str(GKP) + ',' + str(GKI) + ',' + str(GKD) + ',' + str(KForm) + ',' + str(FeedForwardGainVelocity) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerCorrectorPIDFFVelocityGet :  Read corrector parameters
    def PositionerCorrectorPIDFFVelocityGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorPIDFFVelocityGet(' + PositionerName + ',bool *,double *,double *,double *,double *,double *,double *,double *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(12):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerCorrectorPIDDualFFVoltageSet :  Update corrector parameters
    def PositionerCorrectorPIDDualFFVoltageSet (self, socketId, PositionerName, ClosedLoopStatus, KP, KI, KD, KS, IntegrationTime, DerivativeFilterCutOffFrequency, GKP, GKI, GKD, KForm, FeedForwardGainVelocity, FeedForwardGainAcceleration, Friction):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorPIDDualFFVoltageSet(' + PositionerName + ',' + str(ClosedLoopStatus) + ',' + str(KP) + ',' + str(KI) + ',' + str(KD) + ',' + str(KS) + ',' + str(IntegrationTime) + ',' + str(DerivativeFilterCutOffFrequency) + ',' + str(GKP) + ',' + str(GKI) + ',' + str(GKD) + ',' + str(KForm) + ',' + str(FeedForwardGainVelocity) + ',' + str(FeedForwardGainAcceleration) + ',' + str(Friction) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerCorrectorPIDDualFFVoltageGet :  Read corrector parameters
    def PositionerCorrectorPIDDualFFVoltageGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorPIDDualFFVoltageGet(' + PositionerName + ',bool *,double *,double *,double *,double *,double *,double *,double *,double *,double *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(14):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerCorrectorPIPositionSet :  Update corrector parameters
    def PositionerCorrectorPIPositionSet (self, socketId, PositionerName, ClosedLoopStatus, KP, KI, IntegrationTime):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorPIPositionSet(' + PositionerName + ',' + str(ClosedLoopStatus) + ',' + str(KP) + ',' + str(KI) + ',' + str(IntegrationTime) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerCorrectorPIPositionGet :  Read corrector parameters
    def PositionerCorrectorPIPositionGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorPIPositionGet(' + PositionerName + ',bool *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(4):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerCorrectorTypeGet :  Read corrector type
    def PositionerCorrectorTypeGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorTypeGet(' + PositionerName + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerCurrentVelocityAccelerationFiltersSet :  Set current velocity and acceleration cut off frequencies
    def PositionerCurrentVelocityAccelerationFiltersSet (self, socketId, PositionerName, CurrentVelocityCutOffFrequency, CurrentAccelerationCutOffFrequency):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCurrentVelocityAccelerationFiltersSet(' + PositionerName + ',' + str(CurrentVelocityCutOffFrequency) + ',' + str(CurrentAccelerationCutOffFrequency) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerCurrentVelocityAccelerationFiltersGet :  Get current velocity and acceleration cut off frequencies
    def PositionerCurrentVelocityAccelerationFiltersGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCurrentVelocityAccelerationFiltersGet(' + PositionerName + ',double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerDriverStatusGet :  Read positioner driver status
    def PositionerDriverStatusGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerDriverStatusGet(' + PositionerName + ',int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # PositionerDriverStatusStringGet :  Return the positioner driver status string corresponding to the positioner error code
    def PositionerDriverStatusStringGet (self, socketId, PositionerDriverStatus):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerDriverStatusStringGet(' + str(PositionerDriverStatus) + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerEncoderAmplitudeValuesGet :  Read analog interpolated encoder amplitude values
    def PositionerEncoderAmplitudeValuesGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerEncoderAmplitudeValuesGet(' + PositionerName + ',double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(4):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerEncoderCalibrationParametersGet :  Read analog interpolated encoder calibration parameters
    def PositionerEncoderCalibrationParametersGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerEncoderCalibrationParametersGet(' + PositionerName + ',double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(4):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerErrorGet :  Read and clear positioner error code
    def PositionerErrorGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerErrorGet(' + PositionerName + ',int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # PositionerErrorRead :  Read only positioner error code without clear it
    def PositionerErrorRead (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerErrorRead(' + PositionerName + ',int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # PositionerErrorStringGet :  Return the positioner status string corresponding to the positioner error code
    def PositionerErrorStringGet (self, socketId, PositionerErrorCode):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerErrorStringGet(' + str(PositionerErrorCode) + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerHardwareStatusGet :  Read positioner hardware status
    def PositionerHardwareStatusGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerHardwareStatusGet(' + PositionerName + ',int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # PositionerHardwareStatusStringGet :  Return the positioner hardware status string corresponding to the positioner error code
    def PositionerHardwareStatusStringGet (self, socketId, PositionerHardwareStatus):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerHardwareStatusStringGet(' + str(PositionerHardwareStatus) + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerHardInterpolatorFactorGet :  Get hard interpolator parameters
    def PositionerHardInterpolatorFactorGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerHardInterpolatorFactorGet(' + PositionerName + ',int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # PositionerHardInterpolatorFactorSet :  Set hard interpolator parameters
    def PositionerHardInterpolatorFactorSet (self, socketId, PositionerName, InterpolationFactor):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerHardInterpolatorFactorSet(' + PositionerName + ',' + str(InterpolationFactor) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerMaximumVelocityAndAccelerationGet :  Return maximum velocity and acceleration of the positioner
    def PositionerMaximumVelocityAndAccelerationGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerMaximumVelocityAndAccelerationGet(' + PositionerName + ',double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerMotionDoneGet :  Read motion done parameters
    def PositionerMotionDoneGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerMotionDoneGet(' + PositionerName + ',double *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(5):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerMotionDoneSet :  Update motion done parameters
    def PositionerMotionDoneSet (self, socketId, PositionerName, PositionWindow, VelocityWindow, CheckingTime, MeanPeriod, TimeOut):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerMotionDoneSet(' + PositionerName + ',' + str(PositionWindow) + ',' + str(VelocityWindow) + ',' + str(CheckingTime) + ',' + str(MeanPeriod) + ',' + str(TimeOut) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerPositionCompareGet :  Read position compare parameters
    def PositionerPositionCompareGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerPositionCompareGet(' + PositionerName + ',double *,double *,double *,bool *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(4):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerPositionCompareSet :  Set position compare parameters
    def PositionerPositionCompareSet (self, socketId, PositionerName, MinimumPosition, MaximumPosition, PositionStep):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerPositionCompareSet(' + PositionerName + ',' + str(MinimumPosition) + ',' + str(MaximumPosition) + ',' + str(PositionStep) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerPositionCompareEnable :  Enable position compare
    def PositionerPositionCompareEnable (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerPositionCompareEnable(' + PositionerName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerPositionCompareDisable :  Disable position compare
    def PositionerPositionCompareDisable (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerPositionCompareDisable(' + PositionerName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionersEncoderIndexDifferenceGet :  Return the difference between index of primary axis and secondary axis (only after homesearch)
    def PositionersEncoderIndexDifferenceGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionersEncoderIndexDifferenceGet(' + PositionerName + ',double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # PositionerSGammaExactVelocityAjustedDisplacementGet :  Return adjusted displacement to get exact velocity
    def PositionerSGammaExactVelocityAjustedDisplacementGet (self, socketId, PositionerName, DesiredDisplacement):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerSGammaExactVelocityAjustedDisplacementGet(' + PositionerName + ',' + str(DesiredDisplacement) + ',double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # PositionerSGammaParametersGet :  Read dynamic parameters for one axe of a group for a future displacement
    def PositionerSGammaParametersGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerSGammaParametersGet(' + PositionerName + ',double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(4):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerSGammaParametersSet :  Update dynamic parameters for one axe of a group for a future displacement
    def PositionerSGammaParametersSet (self, socketId, PositionerName, Velocity, Acceleration, MinimumTjerkTime, MaximumTjerkTime):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerSGammaParametersSet(' + PositionerName + ',' + str(Velocity) + ',' + str(Acceleration) + ',' + str(MinimumTjerkTime) + ',' + str(MaximumTjerkTime) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerSGammaPreviousMotionTimesGet :  Read SettingTime and SettlingTime
    def PositionerSGammaPreviousMotionTimesGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerSGammaPreviousMotionTimesGet(' + PositionerName + ',double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerStageParameterGet :  Return the stage parameter
    def PositionerStageParameterGet (self, socketId, PositionerName, ParameterName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerStageParameterGet(' + PositionerName + ',' + ParameterName + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerStageParameterSet :  Save the stage parameter
    def PositionerStageParameterSet (self, socketId, PositionerName, ParameterName, ParameterValue):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerStageParameterSet(' + PositionerName + ',' + ParameterName + ',' + ParameterValue + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerTimeFlasherGet :  Read time flasher parameters
    def PositionerTimeFlasherGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerTimeFlasherGet(' + PositionerName + ',double *,double *,double *,bool *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(4):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerTimeFlasherSet :  Set time flasher parameters
    def PositionerTimeFlasherSet (self, socketId, PositionerName, MinimumPosition, MaximumPosition, TimeInterval):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerTimeFlasherSet(' + PositionerName + ',' + str(MinimumPosition) + ',' + str(MaximumPosition) + ',' + str(TimeInterval) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerTimeFlasherEnable :  Enable time flasher
    def PositionerTimeFlasherEnable (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerTimeFlasherEnable(' + PositionerName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerTimeFlasherDisable :  Disable time flasher
    def PositionerTimeFlasherDisable (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerTimeFlasherDisable(' + PositionerName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerUserTravelLimitsGet :  Read UserMinimumTarget and UserMaximumTarget
    def PositionerUserTravelLimitsGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerUserTravelLimitsGet(' + PositionerName + ',double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerUserTravelLimitsSet :  Update UserMinimumTarget and UserMaximumTarget
    def PositionerUserTravelLimitsSet (self, socketId, PositionerName, UserMinimumTarget, UserMaximumTarget):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerUserTravelLimitsSet(' + PositionerName + ',' + str(UserMinimumTarget) + ',' + str(UserMaximumTarget) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerCorrectorAutoTuning :  Astrom&Hagglund based auto-tuning
    def PositionerCorrectorAutoTuning (self, socketId, PositionerName, TuningMode):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerCorrectorAutoTuning(' + PositionerName + ',' + str(TuningMode) + ',double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(3):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # PositionerAccelerationAutoScaling :  Astrom&Hagglund based auto-scaling
    def PositionerAccelerationAutoScaling (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerAccelerationAutoScaling(' + PositionerName + ',double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # MultipleAxesPVTVerification :  Multiple axes PVT trajectory verification
    def MultipleAxesPVTVerification (self, socketId, GroupName, TrajectoryFileName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'MultipleAxesPVTVerification(' + GroupName + ',' + TrajectoryFileName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # MultipleAxesPVTVerificationResultGet :  Multiple axes PVT trajectory verification result get
    def MultipleAxesPVTVerificationResultGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'MultipleAxesPVTVerificationResultGet(' + PositionerName + ',char *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(4):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # MultipleAxesPVTExecution :  Multiple axes PVT trajectory execution
    def MultipleAxesPVTExecution (self, socketId, GroupName, TrajectoryFileName, ExecutionNumber):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'MultipleAxesPVTExecution(' + GroupName + ',' + TrajectoryFileName + ',' + str(ExecutionNumber) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # MultipleAxesPVTParametersGet :  Multiple axes PVT trajectory get parameters
    def MultipleAxesPVTParametersGet (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'MultipleAxesPVTParametersGet(' + GroupName + ',char *,int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # MultipleAxesPVTPulseOutputSet :  Configure pulse output on trajectory
    def MultipleAxesPVTPulseOutputSet (self, socketId, GroupName, StartElement, EndElement, TimeInterval):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'MultipleAxesPVTPulseOutputSet(' + GroupName + ',' + str(StartElement) + ',' + str(EndElement) + ',' + str(TimeInterval) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # MultipleAxesPVTPulseOutputGet :  Get pulse output on trajectory configuration
    def MultipleAxesPVTPulseOutputGet (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'MultipleAxesPVTPulseOutputGet(' + GroupName + ',int *,int *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(3):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # SingleAxisSlaveModeEnable :  Enable the slave mode
    def SingleAxisSlaveModeEnable (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'SingleAxisSlaveModeEnable(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # SingleAxisSlaveModeDisable :  Disable the slave mode
    def SingleAxisSlaveModeDisable (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'SingleAxisSlaveModeDisable(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # SingleAxisSlaveParametersSet :  Set slave parameters
    def SingleAxisSlaveParametersSet (self, socketId, GroupName, PositionerName, Ratio):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'SingleAxisSlaveParametersSet(' + GroupName + ',' + PositionerName + ',' + str(Ratio) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # SingleAxisSlaveParametersGet :  Get slave parameters
    def SingleAxisSlaveParametersGet (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'SingleAxisSlaveParametersGet(' + GroupName + ',char *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # SpindleSlaveModeEnable :  Enable the slave mode
    def SpindleSlaveModeEnable (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'SpindleSlaveModeEnable(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # SpindleSlaveModeDisable :  Disable the slave mode
    def SpindleSlaveModeDisable (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'SpindleSlaveModeDisable(' + GroupName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # SpindleSlaveParametersSet :  Set slave parameters
    def SpindleSlaveParametersSet (self, socketId, GroupName, PositionerName, Ratio):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'SpindleSlaveParametersSet(' + GroupName + ',' + PositionerName + ',' + str(Ratio) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # SpindleSlaveParametersGet :  Get slave parameters
    def SpindleSlaveParametersGet (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'SpindleSlaveParametersGet(' + GroupName + ',char *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
            j += 1
        retList.append(eval(returnedString[i:i+j]))

        return retList


    # GroupSpinParametersSet :  Modify Spin parameters on selected group and activate the continuous move
    def GroupSpinParametersSet (self, socketId, GroupName, Velocity, Acceleration):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupSpinParametersSet(' + GroupName + ',' + str(Velocity) + ',' + str(Acceleration) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupSpinParametersGet :  Get Spin parameters on selected group
    def GroupSpinParametersGet (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupSpinParametersGet(' + GroupName + ',double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GroupSpinCurrentGet :  Get Spin current on selected group
    def GroupSpinCurrentGet (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupSpinCurrentGet(' + GroupName + ',double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # GroupSpinModeStop :  Stop Spin mode on selected group with specified acceleration
    def GroupSpinModeStop (self, socketId, GroupName, Acceleration):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupSpinModeStop(' + GroupName + ',' + str(Acceleration) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # XYLineArcVerification :  XY trajectory verification
    def XYLineArcVerification (self, socketId, GroupName, TrajectoryFileName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'XYLineArcVerification(' + GroupName + ',' + TrajectoryFileName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # XYLineArcVerificationResultGet :  XY trajectory verification result get
    def XYLineArcVerificationResultGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'XYLineArcVerificationResultGet(' + PositionerName + ',char *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(4):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # XYLineArcExecution :  XY trajectory execution
    def XYLineArcExecution (self, socketId, GroupName, TrajectoryFileName, Velocity, Acceleration, ExecutionNumber):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'XYLineArcExecution(' + GroupName + ',' + TrajectoryFileName + ',' + str(Velocity) + ',' + str(Acceleration) + ',' + str(ExecutionNumber) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # XYLineArcParametersGet :  XY trajectory get parameters
    def XYLineArcParametersGet (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'XYLineArcParametersGet(' + GroupName + ',char *,double *,double *,int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(3):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # XYLineArcPulseOutputSet :  Configure pulse output on trajectory
    def XYLineArcPulseOutputSet (self, socketId, GroupName, StartLength, EndLength, PathLengthInterval):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'XYLineArcPulseOutputSet(' + GroupName + ',' + str(StartLength) + ',' + str(EndLength) + ',' + str(PathLengthInterval) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # XYLineArcPulseOutputGet :  Get pulse output on trajectory configuration
    def XYLineArcPulseOutputGet (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'XYLineArcPulseOutputGet(' + GroupName + ',double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(3):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # XYZSplineVerification :  XYZ trajectory verifivation
    def XYZSplineVerification (self, socketId, GroupName, TrajectoryFileName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'XYZSplineVerification(' + GroupName + ',' + TrajectoryFileName + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # XYZSplineVerificationResultGet :  XYZ trajectory verification result get
    def XYZSplineVerificationResultGet (self, socketId, PositionerName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'XYZSplineVerificationResultGet(' + PositionerName + ',char *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(4):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # XYZSplineExecution :  XYZ trajectory execution
    def XYZSplineExecution (self, socketId, GroupName, TrajectoryFileName, Velocity, Acceleration):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'XYZSplineExecution(' + GroupName + ',' + TrajectoryFileName + ',' + str(Velocity) + ',' + str(Acceleration) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # XYZSplineParametersGet :  XYZ trajectory get parameters
    def XYZSplineParametersGet (self, socketId, GroupName):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'XYZSplineParametersGet(' + GroupName + ',char *,double *,double *,int *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(3):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # EEPROMCIESet :  XYZ trajectory get parameters
    def EEPROMCIESet (self, socketId, CardNumber, ReferenceString):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EEPROMCIESet(' + str(CardNumber) + ',' + ReferenceString + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EEPROMDACOffsetCIESet :  XYZ trajectory get parameters
    def EEPROMDACOffsetCIESet (self, socketId, PlugNumber, DAC1Offset, DAC2Offset):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EEPROMDACOffsetCIESet(' + str(PlugNumber) + ',' + str(DAC1Offset) + ',' + str(DAC2Offset) + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EEPROMDriverSet :  XYZ trajectory get parameters
    def EEPROMDriverSet (self, socketId, PlugNumber, ReferenceString):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EEPROMDriverSet(' + str(PlugNumber) + ',' + ReferenceString + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EEPROMINTSet :  XYZ trajectory get parameters
    def EEPROMINTSet (self, socketId, CardNumber, ReferenceString):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EEPROMINTSet(' + str(CardNumber) + ',' + ReferenceString + ')'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # CPUCoreAndBoardSupplyVoltagesGet :  XYZ trajectory get parameters
    def CPUCoreAndBoardSupplyVoltagesGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'CPUCoreAndBoardSupplyVoltagesGet(double *,double *,double *,double *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(8):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # CPUTemperatureAndFanSpeedGet :  XYZ trajectory get parameters
    def CPUTemperatureAndFanSpeedGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'CPUTemperatureAndFanSpeedGet(double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(2):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # ActionListGet :  Action list
    def ActionListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'ActionListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # ActionExtendedListGet :  Action extended list
    def ActionExtendedListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'ActionExtendedListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # APIExtendedListGet :  API method list
    def APIExtendedListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'APIExtendedListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # APIListGet :  API method list without extended API
    def APIListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'APIListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # ErrorListGet :  Error list
    def ErrorListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'ErrorListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # EventListGet :  General event list
    def EventListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'EventListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringListGet :  Gathering type list
    def GatheringListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringExtendedListGet :  Gathering type extended list
    def GatheringExtendedListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringExtendedListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringExternalListGet :  External Gathering type list
    def GatheringExternalListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringExternalListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GroupStatusListGet :  Group status list
    def GroupStatusListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GroupStatusListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # HardwareInternalListGet :  Internal hardware list
    def HardwareInternalListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'HardwareInternalListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # HardwareDriverAndStageGet :  Smart hardware
    def HardwareDriverAndStageGet (self, socketId, PlugNumber):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'HardwareDriverAndStageGet(' + str(PlugNumber) + ',char *,char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # ObjectsListGet :  Group name and positioner name
    def ObjectsListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'ObjectsListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerErrorListGet :  Positioner error list
    def PositionerErrorListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerErrorListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerHardwareStatusListGet :  Positioner hardware status list
    def PositionerHardwareStatusListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerHardwareStatusListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # PositionerDriverStatusListGet :  Positioner driver status list
    def PositionerDriverStatusListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'PositionerDriverStatusListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # ReferencingActionListGet :  Get referencing action list
    def ReferencingActionListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'ReferencingActionListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # ReferencingSensorListGet :  Get referencing sensor list
    def ReferencingSensorListGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'ReferencingSensorListGet(char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


    # GatheringUserDatasGet :  Return UserDatas values
    def GatheringUserDatasGet (self, socketId):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'GatheringUserDatasGet(double *,double *,double *,double *,double *,double *,double *,double *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        if (error != 0):
            return [error, returnedString]

        i, j, retList = 0, 0, [error]
        for paramNb in range(8):
            while ((i+j) < len(returnedString) and returnedString[i+j] != ','):
                j += 1
            retList.append(eval(returnedString[i:i+j]))
            i, j = i+j+1, 0

        return retList


    # TestTCP :  Test TCP/IP transfert
    def TestTCP (self, socketId, InputString):
        if (XPS.__usedSockets[socketId] == 0):
            return

        command = 'TestTCP(' + InputString + ',char *)'
        [error, returnedString] = self.__sendAndReceive(socketId, command)
        return [error, returnedString]


#if __name__ == '__main__' :
#
#    my_xps = XPS()
#    my_socketId = my_xps.TCP_ConnectToServer('192.168.0.254',5001,1)
#    output=my_xps.GroupPositionCurrentGet(my_socketId,'NSR1',1)[1]
#    my_xps.GroupMoveAbsolute(my_socketId,'NSR1',[15])
#    output=my_xps.GroupPositionCurrentGet(my_socketId,'NSR1',1)[1]
#    my_xps.TCP_CloseSocket(my_socketId)
#    print(output)

    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Function generator'

class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Examples:  ----------------

usage:    autolab driver [options] args
            
    autolab driver -D {self.name} -C VXI11 -A 192.168.0.4 -f 1000000 -p 0 -o 0.5 -a 1 -d 10e-6 -c AB,EF
    load {self.name} driver using VXI11 and local network address 192.168.0.4 and sets the frequency to 1MHz and acts on outputs AB and EF to set the polarity to 0 the offset to 0.5V amplitude to 1V and the delay to 10e-6s.
    
    autolab driver -D nickname -f 80e6 -c A,B -a 0.5 
    Similar to previous one but using the device's nickname as defined in local_config.ini and only for channel A and B. This will act only on B for instance if you do precise only B as channel argument.
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-f", "--frequency", type=str, dest="frequency", default=None, help="Set the frequency. This is applied to all of the outputs" )
        parser.add_argument("-c", "--channels", type=str, dest="channels", default=None, help="Set the channels to act on/acquire from. channels will be a list of outputs('AB','CD','EF','GH') or independant channels ('A','B','C','D','E','F','G','H'). WARNING: you can only change the relative delay between independant channels, you must act on outputs for other options. Pre-configured options only allow you to act on either outputs[single('AB') or several('AB','CD','GH')] or channels[single('A') or couple('A','B') only] with a single command => use several commands or -m option instead." )
        parser.add_argument("-a", "--amplitude", type=str, dest="amplitude", default=None, help="Set the amplitude.")
        parser.add_argument("-p", "--polarity", type=str, dest="polarity", default=None, help="Set the level polarity if 1, then up if 0, then down")
        parser.add_argument("-o", "--offset", type=str, dest="offset", default=None, help="Set the offset.")
        parser.add_argument("-d", "--delay", type=str, dest="delay", default=None, help="Set the delay (s).")
        
        return parser

    def do_something(self,args):
        if args.frequency:
            getattr(self.Instance,'set_frequency')(args.frequency)
        if args.delay and len(args.channels[0])==1:  # modify the delay for channels 'A','B,...'
            getattr(self.Instance,'set_delay_channels')(args.channels)
            getattr(self.Instance,'set_delay')(args.delay)
        elif args.channels:
            for out in args.channels.split(','):
                if args.amplitude:
                    getattr(getattr(self.Instance,f'output{out}'),'set_amplitude')(args.amplitude)
                if args.polarity:
                    getattr(getattr(self.Instance,f'output{out}'),'set_polarity')(args.polarity)
                if args.offset:
                    getattr(getattr(self.Instance,f'output{out}'),'set_offset')(args.offset)
                if args.delay:
                    getattr(getattr(self.Instance,f'output{out}'),'set_delay')(args.delay)


    def exit(self):
        self.Instance.close()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Optical source'

class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Examples:  ----------------

usage:    autolab driver [options] args
            
    autolab driver -D {self.name} -C USB -w 1550.2 -p 10
    load {self.name} driver using USB communication protocol with preconfigured vendor and deviceID and set the wavelength to 1550.2nm and the piezo to 10%.
    
    autolab driver -D nickname -x 1550.2,1551.3,3
    Use now the device nickname as defined in local_config.ini and perform a wavelength scan from 1550.2nm to 1551.3nm at a velocity of 3nm/s. I no third argument specified will use the device's stored one.

    autolab driver -D nickname -r 10,30.3,3
    Use now the device nickname as defined in local_config.ini and perform a piezo scan from 10% to 30% with a total period of 3s. If no third argument specified will go as fast as possible.
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-w", "--wavelength", type=str, dest="wavelength", default=None, help="Set the wavelength value." )
        parser.add_argument("-x", "--scan_wavelength", type=str, dest="scan_wavelength", default=None, help="Set the wavelength scan values and start a scan. Arguments are:  min,max,speed (1 is the start wavelength, 2 is the stop wavelength, 3 is the scan speed in nm/s)." )
        parser.add_argument("-p", "--piezo", type=str, dest="piezo", default=None, help="Set the voltage to apply to the piezo in percent." )
        parser.add_argument("-r", "--scan_piezo", type=str,dest="scan_piezo", default=None, help="Set the values for the piezo scan and start a scan. Arguments are: min,max,time_scan (1 is the piezo start value, 2 is the piezo stop value, 3 is the total scan period)." )
        
        return parser

    def do_something(self,args):
        if args.wavelength:
            getattr(self.Instance,'wavelength')(args.wavelength)
        if args.piezo:
            getattr(self.Instance,'set_piezo')(args.piezo)
        if args.scan_wavelength:
            scan_wavelength = args.scan_wavelength.split(',')
            if len(scan_wavelength) == 3:
                getattr(self.Instance,'set_scan_forward_velocity')(scan_wavelength[2])
                getattr(self.Instance,'set_scan_backward_velocity')(scan_wavelength[2])
            elif len(scan_wavelength)!=2:
                print('Please provide 2 or 3 values for the scan')
                sys.exit()
            getattr(self.Instance,'set_scan_start_wavelength')(scan_wavelength[0])
            getattr(self.Instance,'set_scan_stop_wavelength')(scan_wavelength[1])
            getattr(self.Instance,'start_scan_wavelength')()  # starts wavelength scan
            sys.exit()
        if args.scan_piezo:
            getattr(self.Instance,'ramp_scanpiezo')(args.scan_piezo.split(','))
            sys.exit()


    def exit(self):
        #self.Instance.close()
        pass



# -*- coding: utf-8 -*-
"""
Created on Tue Mar 28 14:48:14 2017

@author: Quentin Chateiller
quentin.chateiller@c2n.upsaclay.fr


"""
#from threading import Event, Thread

#started = Event()

#def gui() :
#        
#    if started.is_set():
#        print("Gui already running")
#    else :
#        t=AppThread()
#        t.start()


def start():
    
    from PyQt5 import QtWidgets
    
    app = QtWidgets.QApplication.instance()
    if app is None:
        app = QtWidgets.QApplication([])
    
    from .controlcenter.main import ControlCenter
    gui = ControlCenter()
    gui.initialize()
    gui.show()
    app.exec_()
        
#class AppThread(Thread):
#    
#    def __init__(self):
#        
#        Thread.__init__(self)
#            
#    def run(self):
#                
#        started.set()
#        
#        _run()
#        
#        started.clear()
#        
            


# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:14:28 2019

@author: qchat
"""
import numpy as np
import math as m

class RangeManager : 
    
    def __init__(self,gui):
        
        self.gui = gui
        
        # Widget 'return pressed' signal connections
        self.gui.scanLog_checkBox.stateChanged.connect(self.scanLogChanged)
        self.gui.nbpts_lineEdit.returnPressed.connect(self.nbptsChanged)
        self.gui.step_lineEdit.returnPressed.connect(self.stepChanged)
        self.gui.start_lineEdit.returnPressed.connect(self.startChanged)
        self.gui.end_lineEdit.returnPressed.connect(self.endChanged)
        self.gui.mean_lineEdit.returnPressed.connect(self.meanChanged)
        self.gui.width_lineEdit.returnPressed.connect(self.widthChanged)
        
        # Widget 'text edited' signal connections
        self.gui.nbpts_lineEdit.textEdited.connect(lambda : self.gui.setLineEditBackground(self.gui.nbpts_lineEdit,'edited'))
        self.gui.step_lineEdit.textEdited.connect(lambda : self.gui.setLineEditBackground(self.gui.step_lineEdit,'edited'))
        self.gui.start_lineEdit.textEdited.connect(lambda : self.gui.setLineEditBackground(self.gui.start_lineEdit,'edited'))
        self.gui.end_lineEdit.textEdited.connect(lambda : self.gui.setLineEditBackground(self.gui.end_lineEdit,'edited'))
        self.gui.mean_lineEdit.textEdited.connect(lambda : self.gui.setLineEditBackground(self.gui.mean_lineEdit,'edited'))
        self.gui.width_lineEdit.textEdited.connect(lambda : self.gui.setLineEditBackground(self.gui.width_lineEdit,'edited'))
        
        # Push button
        self.gui.fromFigure_pushButton.clicked.connect(self.fromFigureButtonClicked)
        self.gui.fromFigure_pushButton.setEnabled(False)
        
        self.refresh()
                        
        
        
    def fromFigureButtonClicked(self):
        
        """ This function sets the start and end value, and log state, of the scan from currrent figure range """
        
        xrange = self.gui.figureManager.getRange('x')
        self.gui.configManager.setRange(xrange)
        
        log = self.gui.figureManager.isLogScaleEnabled('x')
        self.gui.configManager.setLog(log)
        
        
        
    def refresh(self):
        
        """ This function refreshes all the values displayed of the scan configuration
        from the configuration center"""
        
        xrange = self.gui.configManager.getRange()
        
        # Start
        start = xrange[0]
        self.gui.start_lineEdit.setText(f'{start:g}')
        self.gui.setLineEditBackground(self.gui.start_lineEdit,'synced')
        
        # End
        end = xrange[1]
        self.gui.end_lineEdit.setText(f'{end:g}')
        self.gui.setLineEditBackground(self.gui.end_lineEdit,'synced')
        
        # Mean
        mean = (start+end)/2
        self.gui.mean_lineEdit.setText(f'{mean:g}')
        self.gui.setLineEditBackground(self.gui.mean_lineEdit,'synced')
        
        # Width        
        width = abs(end-start)
        self.gui.width_lineEdit.setText(f'{width:g}')
        self.gui.setLineEditBackground(self.gui.width_lineEdit,'synced')
        
        # Nbpts
        nbpts = self.gui.configManager.getNbPts()
        self.gui.nbpts_lineEdit.setText(f'{nbpts:g}')
        self.gui.setLineEditBackground(self.gui.nbpts_lineEdit,'synced')
        
        # Log 
        log = self.gui.configManager.getLog()
        self.gui.scanLog_checkBox.setChecked(log)
        
        # Step
        if log is False :
            step = width / (nbpts-1)
            self.gui.step_lineEdit.setText(f'{step:g}')
            self.gui.step_lineEdit.setEnabled(True)
        else :
            self.gui.step_lineEdit.setEnabled(False)
            self.gui.step_lineEdit.setText('')
        self.gui.setLineEditBackground(self.gui.step_lineEdit,'synced')
            
            
        
    def nbptsChanged(self) :
        
        """ This function changes the number of point of the scan """
        
        value = self.gui.nbpts_lineEdit.text()
        
        try :
            value = int(float(value))
            assert value > 2
            self.gui.configManager.setNbPts(value)
        except :
            self.refresh()
        
        
        
    def stepChanged(self) :
        
        """ This function changes the step size of the scan """
        
        value = self.gui.step_lineEdit.text()

        try :
            value = float(value)
            assert value > 0
            xrange = list(self.gui.configManager.getRange())
            width = xrange[1]-xrange[0]
            nbpts = round(abs(width)/value)+1
            self.gui.configManager.setNbPts(nbpts)
            xrange[1] = xrange[0]+np.sign(width)*(nbpts-1)*value
            self.gui.configManager.setRange(xrange)
        except :
            self.refresh()
        
        
        
    def startChanged(self) : 
        
        """ This function changes the start value of the scan """
        
        value = self.gui.start_lineEdit.text()
        
        try:
            value=float(value)
            log = self.gui.configManager.getLog()
            if log is True : assert value>0
            print(value,log,'pas de soucis')
            xrange = list(self.gui.configManager.getRange())
            xrange[0] = value
            self.gui.configManager.setRange(xrange)
        except:
            self.refresh()



    def endChanged(self) : 
        
        """ This function changes the end value of the scan """
        
        value = self.gui.end_lineEdit.text()
        
        try:
            value=float(value)
            log = self.gui.configManager.getLog()
            if log is True : assert value>0
            xrange = list(self.gui.configManager.getRange())
            xrange[1] = value
            self.gui.configManager.setRange(xrange)
        except :
            self.refresh()



    def meanChanged(self) : 
        
        """ This function changes the mean value of the scan """
        
        value = self.gui.mean_lineEdit.text()
        
        try:
            value=float(value)
            log = self.gui.configManager.getLog()
            if log is True : assert value>0
            xrange = list(self.gui.configManager.getRange())
            xrange_new = xrange.copy()
            xrange_new[0] = value - (xrange[1]-xrange[0])/2
            xrange_new[1] = value + (xrange[1]-xrange[0])/2
            assert xrange_new[0] > 0
            assert xrange_new[1] > 0
            self.gui.configManager.setRange(xrange_new)
        except:
            self.refresh()
     
        
        
    def widthChanged(self) : 
        
        """ This function changes the width of the scan """
        
        value = self.gui.width_lineEdit.text()
        
        try:
            value=float(value)
            log = self.gui.configManager.getLog()
            if log is True : assert value>0
            xrange = list(self.gui.configManager.getRange())
            xrange_new = xrange.copy()
            xrange_new[0] = (xrange[1]+xrange[0])/2 - value/2
            xrange_new[1] = (xrange[1]+xrange[0])/2 + value/2  
            assert xrange_new[0] > 0
            assert xrange_new[1] > 0
            self.gui.configManager.setRange(xrange_new)
        except:
            self.refresh()
        
        
        
    def scanLogChanged(self):
        
        """ This function changes the log state of the scan """
        
        state = self.gui.scanLog_checkBox.isChecked()
        if state is True :
            xrange = list(self.gui.configManager.getRange())
            change = False
            if xrange[1] <= 0 : 
                xrange[1] = 1
                change = True
            if xrange[0] <= 0 : 
                xrange[0] = 10**(m.log10(xrange[1])-1)
                change = True
            if change is True : 
                self.gui.configManager.setRange(xrange)
        
        self.gui.configManager.setLog(state)
                


# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:12:24 2019

@author: qchat
"""


import os
import math as m
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
import matplotlib

class FigureManager :    
    
    def __init__(self,gui):
        
        self.gui = gui
        
        self.curves = []
        
        # Configure and initialize the figure in the GUI
        self.fig = Figure()
        matplotlib.rcParams.update({'font.size': 12})
        self.ax = self.fig.add_subplot(111)  
        self.ax.grid()
        self.ax.set_xlim((0,10))
        self.ax.autoscale(enable=False)
        # The first time we open a monitor it doesn't work, I don't know why..
        # There is no current event loop in thread 'Thread-7'.
        # More accurately, FigureCanvas doesn't find the event loop the first time it is called
        # The second time it works..
        # Seems to be only in Spyder..
        try : 
            self.canvas = FigureCanvas(self.fig) 
        except :
            self.canvas = FigureCanvas(self.fig)
        self.gui.graph.addWidget(self.canvas)
        self.fig.tight_layout()
        self.canvas.draw()
        
        for axe in ['x','y'] :
            getattr(self.gui,f'logScale_{axe}_checkBox').stateChanged.connect(lambda b, axe=axe:self.logScaleChanged(axe))
            getattr(self.gui,f'autoscale_{axe}_checkBox').stateChanged.connect(lambda b, axe=axe:self.autoscaleChanged(axe))
            getattr(self.gui,f'autoscale_{axe}_checkBox').setChecked(True)
            getattr(self.gui,f'zoom_{axe}_pushButton').clicked.connect(lambda b,axe=axe:self.zoomButtonClicked('zoom',axe))
            getattr(self.gui,f'unzoom_{axe}_pushButton').clicked.connect(lambda b,axe=axe:self.zoomButtonClicked('unzoom',axe))
            getattr(self.gui,f'variable_{axe}_comboBox').currentIndexChanged.connect(self.variableChanged)
                    
        self.gui.goUp_pushButton.clicked.connect(lambda:self.moveButtonClicked('up'))
        self.gui.goDown_pushButton.clicked.connect(lambda:self.moveButtonClicked('down'))
        self.gui.goLeft_pushButton.clicked.connect(lambda:self.moveButtonClicked('left'))
        self.gui.goRight_pushButton.clicked.connect(lambda:self.moveButtonClicked('right'))
        
        self.fig.canvas.mpl_connect('button_press_event', self.onPress)
        self.fig.canvas.mpl_connect('scroll_event', self.onScroll)
        self.fig.canvas.mpl_connect('motion_notify_event', self.onMotion)
        self.fig.canvas.mpl_connect('button_release_event', self.onRelease)
        self.press = None
        
        self.movestep = 0.05

        # Number of traces
        self.nbtraces = 5
        self.gui.nbTraces_lineEdit.setText(f'{self.nbtraces:g}')
        self.gui.nbTraces_lineEdit.returnPressed.connect(self.nbTracesChanged)
        self.gui.nbTraces_lineEdit.textEdited.connect(lambda : self.gui.setLineEditBackground(self.gui.nbTraces_lineEdit,'edited'))
        self.gui.setLineEditBackground(self.gui.nbTraces_lineEdit,'synced')
        

        
        
        
    
        
    # AUTOSCALING
    ###########################################################################
    
    def autoscaleChanged(self,axe):
        
        """ Set or unset the autoscale mode for the given axis """
        state = self.isAutoscaleEnabled(axe)
        getattr(self.ax,f'set_autoscale{axe}_on')(state)
        if state is True :
            self.doAutoscale(axe)
            self.redraw()
            
            
            
    def isAutoscaleEnabled(self,axe):
        
        """ This function returns True or False whether the autoscale for the given axis
        is enabled """
        
        return getattr(self.gui,f'autoscale_{axe}_checkBox').isChecked()
    
    
    
    def doAutoscale(self,axe):
        
        """ This function proceeds to an autoscale operation of the given axis """
        
        datas = [getattr(curve,f'get_{axe}data')() for curve in self.curves]
        if len(datas)>0 :
            min([min(data) for data in datas])
            minValue = min([min(data) for data in datas])
            maxValue = max([max(data) for data in datas])
            if (minValue,maxValue) != self.getRange(axe) :
                self.setRange(axe,(minValue,maxValue))


            
            
            
    # LOGSCALING
    ###########################################################################

    def logScaleChanged(self,axe):
        
        """ This function is called when the log scale state is changed in the GUI. """
        
        state = getattr(self.gui,f'logScale_{axe}_checkBox').isChecked()
        self.setLogScale(axe,state)
        self.redraw()
        
        
    
    def isLogScaleEnabled(self,axe):
        
        """ This function returns True or False whether the log scale is enabled
        in the given axis """
        
        return getattr(self.ax,f'get_{axe}scale')() == 'log'

            
    
    def setLogScale(self,axe,state):
        
        """ This functions sets or not the log scale for the given axis """
        
        if state is True :
            scaleType = 'log'
        else :
            scaleType = 'linear'
        
        self.checkLimPositive(axe)
        self.ax.grid(state,which='minor',axis=axe)
        getattr(self.ax,f'set_{axe}scale')(scaleType)
        
        
        
    def checkLimPositive(self,axe):
        
        """ This function updates the current range of the given axis to be positive, 
        in case we enter in a log scale mode """
        
        axeRange = list(self.getRange(axe))
            
        change = False
        if axeRange[1] <= 0 : 
            axeRange[1] = 1
            change = True
        if axeRange[0] <= 0 : 
            axeRange[0] = 10**(m.log10(axeRange[1])-1)
            change = True
            
        if change is True :
            self.setRange(axe,axeRange)
        
        
        
        
        
        
    # AXE LABEL
    ###########################################################################
        
    def setLabel(self,axe,value):
        
        """ This function changes the label of the given axis """
        
        getattr(self.ax,f'set_{axe}label')(value)






    # PLOT DATA
    ###########################################################################

            
    def clearData(self):
        
        """ This function removes any plotted curves """
        
        for curve in self.curves :
            curve.remove()
        self.curves = []
        self.redraw()
            
        
        
    def reloadData(self):
        
        ''' This function removes any plotted curves and reload all required curves from 
        data available in the data manager'''
        
        # Remove all curves
        self.clearData()
        
        # Get current displayed result
        variable_x = self.gui.variable_x_comboBox.currentText()
        variable_y = self.gui.variable_y_comboBox.currentText()
        
        # Label update
        self.setLabel('x',variable_x)
        self.setLabel('y',variable_y)
        
        # Load the last results data
        try :
            data = self.gui.dataManager.getData(self.nbtraces,[variable_x,variable_y])       
        except :
            data = None
        
        # Plot them
        if data is not None :
            
            for i in range(len(data)) :
                                
                # Data
                subdata = data[i]
                subdata = subdata.astype(float)
                x = subdata.loc[:,variable_x]
                y = subdata.loc[:,variable_y]
                
                # Apprearance:    
                if i == (len(data)-1) :      
                    color = 'r'
                    alpha = 1
                else:
                    color = 'k'
                    alpha = (self.nbtraces-(len(data)-1-i))/self.nbtraces
                
                # Plot
                curve = self.ax.plot(x,y,'x-',color=color,alpha=alpha)[0]
                self.curves.append(curve)
            
            # Autoscale
            if self.isAutoscaleEnabled('x') is True : self.doAutoscale('x')
            if self.isAutoscaleEnabled('y') is True : self.doAutoscale('y')
            
            self.redraw()
        
        
        
    def reloadLastData(self):
        
        ''' This functions update the data of the last curve '''
        
        # Get current displayed result
        variable_x = self.gui.variable_x_comboBox.currentText()
        variable_y = self.gui.variable_y_comboBox.currentText()
        
        data = self.gui.dataManager.getData(1,[variable_x,variable_y])[0]
        data = data.astype(float)
        
        # Update plot data
        self.curves[-1].set_xdata(data.loc[:,variable_x])
        self.curves[-1].set_ydata(data.loc[:,variable_y])
        
        # Autoscale
        if self.isAutoscaleEnabled('x') is True : self.doAutoscale('x')
        if self.isAutoscaleEnabled('y') is True : self.doAutoscale('y')
        
        self.redraw()
        
        
        
    def variableChanged(self,index):
        
        """ This function is called when the displayed result has been changed in
        the combo box. It proceeds to the change. """
        
        self.clearData()        
        
        if self.gui.variable_x_comboBox.currentIndex() != -1 and self.gui.variable_y_comboBox.currentIndex() != -1 : 
            self.reloadData()
            
        if self.gui.variable_x_comboBox.currentText() == self.gui.configManager.getParameterName() :
            self.gui.fromFigure_pushButton.setEnabled(True)
        else :
            self.gui.fromFigure_pushButton.setEnabled(False)

            
            
        
    # TRACES
    ###########################################################################      
        
    def nbTracesChanged(self):
        
        """ This function is called when the number of traces displayed has been changed
        in the GUI. It proceeds to the change and update the plot. """
                
        value = self.gui.nbTraces_lineEdit.text()
        
        check = False
        try :
            value = int(float(value))
            assert value > 0
            self.nbtraces = value
            check = True
        except :
            pass
        
        self.gui.nbTraces_lineEdit.setText(f'{self.nbtraces:g}')
        self.gui.setLineEditBackground(self.gui.nbTraces_lineEdit,'synced')
        
        if check is True and self.gui.variable_y_comboBox.currentIndex() != -1 :
            self.reloadData()
        
        
    


    # ZOOM UNZOOM BUTTONS
    ###########################################################################

    def zoomButtonClicked(self,action,axe):
        
        """ This function is called when a zoom/unzoom button of a given axis has been pressed.
        It proceeds to the change to the figure range """

        logState = self.isLogScaleEnabled(axe)
        inf,sup = self.getRange(axe)
        
        if logState is False :
            if action == 'zoom' :
                inf_new = inf + (sup-inf)*self.movestep
                sup_new = sup - (sup-inf)*self.movestep
            elif action == 'unzoom' :
                inf_new = inf - (sup-inf)*self.movestep/(1-2*self.movestep)
                sup_new = sup + (sup-inf)*self.movestep/(1-2*self.movestep)
        else :
            log_inf = m.log10(inf)
            log_sup = m.log10(sup)
            if action == 'zoom' :
                inf_new = 10**(log_inf+(log_sup-log_inf)*self.movestep)
                sup_new = 10**(log_sup-(log_sup-log_inf)*self.movestep)
            elif action == 'unzoom' :
                inf_new = 10**(log_inf-(log_sup-log_inf)*self.movestep/(1-2*self.movestep))
                sup_new = 10**(log_sup+(log_sup-log_inf)*self.movestep/(1-2*self.movestep))
                
        self.setRange(axe,(inf_new,sup_new))
        self.redraw()




    # MOVE BUTTONS
    ###########################################################################
    
    def moveButtonClicked(self,action):
        
        """ This function is called when a move button of a given axis has been pressed.
        It proceeds to the change to the figure range """
        
        if action in ['up','down'] :
            axe = 'y'
            if action == 'up' : action = 'increase'
            elif action == 'down' : action = 'decrease'
        elif action in ['left','right'] :
            axe = 'x'
            if action == 'left' : action = 'decrease'
            elif action == 'right' : action = 'increase'

        logState = self.isLogScaleEnabled(axe)
            
        inf,sup = self.getRange(axe)
        
        if logState is False :
            if action == 'increase' :
                inf_new = inf + (sup-inf)*self.movestep
                sup_new = sup + (sup-inf)*self.movestep
            elif action == 'decrease' :
                inf_new = inf - (sup-inf)*self.movestep
                sup_new = sup - (sup-inf)*self.movestep
        else :
            log_inf = m.log10(inf)
            log_sup = m.log10(sup)
            if action == 'increase' :
                inf_new = 10**(log_inf+(log_sup-log_inf)*self.movestep)
                sup_new = 10**(log_sup+(log_sup-log_inf)*self.movestep)
            elif action == 'decrease' :
                inf_new = 10**(log_inf-(log_sup-log_inf)*self.movestep)
                sup_new = 10**(log_sup-(log_sup-log_inf)*self.movestep)
            
        self.setRange(axe,(inf_new,sup_new))
        self.redraw()
   
    
    
    
        
    # MOUSE SCROLL
    ###########################################################################
     
    def onScroll(self,event):
        
        """ This function is called when the scrolling button of the mouse has been
        actioned for a given axis has been pressed.
        It proceeds to the change to the figure range """
        
        data = {'x':event.xdata,'y':event.ydata}
        
        for axe in ['x','y'] :
            
            logState = self.isLogScaleEnabled(axe)
            inf,sup = self.getRange(axe)
            
            if logState :
            
                if event.button == 'up' : 
                    inf_new = 10**(m.log10(data[axe]) * self.movestep + (1-self.movestep) * m.log10(inf)) 
                    sup_new = 10**(m.log10(data[axe]) * self.movestep + (1-self.movestep) * m.log10(sup)) 
                elif event.button == 'down' :
                    inf_new = 10**( ( - m.log10(data[axe]) * self.movestep + m.log10(inf) ) / (1 - self.movestep) )
                    sup_new = 10**( ( - m.log10(data[axe]) * self.movestep + m.log10(sup) ) / (1 - self.movestep) )
                    
            else :
            
                if event.button == 'up' : 
                    inf_new = data[axe] * self.movestep + (1-self.movestep) * inf 
                    sup_new = data[axe] * self.movestep + (1-self.movestep) * sup 
                elif event.button == 'down' :
                    inf_new = ( - data[axe] * self.movestep + inf ) / (1 - self.movestep)
                    sup_new = ( - data[axe] * self.movestep + sup ) / (1 - self.movestep)   
                
            self.setRange(axe,(inf_new,sup_new))
            self.redraw()
            
            
            
            
        
    # MOUSE DRAG
    ###########################################################################
        
    def onPress(self,event):
        
        """ This function is called when the main button of the mouse is pressed on the figure.
        It saves information about this location, in order to use it in the onMotion function. """
        
        xlim = self.getRange('x')
        ylim = self.getRange('y')
        
        if self.isLogScaleEnabled('x') is False :
            x_width_data = xlim[1]-xlim[0]
        else :
            x_width_data = m.log10(xlim[1])-m.log10(xlim[0])
        
        if self.isLogScaleEnabled('y') is False :
            y_width_data = ylim[1]-ylim[0]
        else :
            y_width_data = m.log10(ylim[1])-m.log10(ylim[0])
        
        leftInfCornerCoords = self.ax.transData.transform((xlim[0],ylim[0]))
        rightSupCornerCoords = self.ax.transData.transform((xlim[1],ylim[1]))       
        x_width_pixel = rightSupCornerCoords[0] - leftInfCornerCoords[0]
        y_width_pixel = rightSupCornerCoords[1] - leftInfCornerCoords[1]
    
        self.press = xlim,ylim,x_width_data,y_width_data,x_width_pixel,y_width_pixel,event.x,event.y

        

    def onMotion(self,event):
        
        """ This function is called when the mouse if moved above the figure. If its button is pressed, 
        proceeds to the corresponding change in range with the help on the data saved with 
        the onPress function. """
        
        if self.press is not None and event.inaxes is not None :
            
            xlim,ylim,x_width_data,y_width_data,x_width_pixel,y_width_pixel,x_press_pixel, y_press_pixel = self.press

            x_pixel,y_pixel = event.x,event.y
            
            xlim_new = list(xlim)
            ylim_new = list(ylim)
            
            dx_pixel = x_pixel - x_press_pixel
            dy_pixel = y_pixel - y_press_pixel
            
            dx_data = dx_pixel * x_width_data / x_width_pixel
            dy_data = dy_pixel * y_width_data / y_width_pixel
            
            if self.isLogScaleEnabled('x'):
                xlim_new[0] = 10**(m.log10(xlim[0])-dx_data)
                xlim_new[1] = 10**(m.log10(xlim[1])-dx_data)
            else :
                xlim_new[0] = xlim[0]-dx_data
                xlim_new[1] = xlim[1]-dx_data
  
            if self.isLogScaleEnabled('y'):
                ylim_new[0] = 10**(m.log10(ylim[0])-dy_data)
                ylim_new[1] = 10**(m.log10(ylim[1])-dy_data)
            else :
                ylim_new[0] = ylim[0]-dy_data
                ylim_new[1] = ylim[1]-dy_data
            
            self.setRange('x',xlim_new)
            self.setRange('y',ylim_new)
            self.redraw()
            
            
    def onRelease(self,event):
        
        """ This function is called when the wiew change using the mouse is finshed.
        It clears the data of the initial press position """
        
        self.press = None
        

        

    
    # SAVE FIGURE
    ###########################################################################
        
    def save(self,path):
        
        """ This function save the figure in the provided path """
        
        self.fig.savefig(os.path.join(path,'figure.jpg'),dpi=300)
        
        
        
        
        
    # redraw
    ###########################################################################

    def redraw(self):
        
        """ This function make the previous changes appears in the GUI """
        
        try :
            self.fig.tight_layout()
        except :
            pass
        self.canvas.draw()
        
        
        
        
    # RANGE
    ###########################################################################
        
    def getRange(self,axe):
        
        """ This function returns the current range of the given axis """
        
        return getattr(self.ax,f'get_{axe}lim')()
    
    
    
    def setRange(self,axe,r):
        
        """ This function sets the current range of the given axis """
        
        if r[0] != r[1] :
            getattr(self.ax,f'set_{axe}lim')(r)
        


# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:08:45 2019

@author: qchat
"""

from PyQt5 import QtCore
import time
import threading
import numpy as np
import collections
import math as m

class ScanManager :
    
    def __init__(self,gui):
        self.gui = gui
        
        # Start / Stop button configuration
        self.gui.start_pushButton.clicked.connect(self.startButtonClicked)
        
        # Pause / Resume button configuration
        self.gui.pause_pushButton.clicked.connect(self.pauseButtonClicked)
        self.gui.pause_pushButton.setEnabled(False)
        
        # Progress bar configuration        
        self.gui.progressBar.setMinimum(0)
        self.gui.progressBar.setValue(0)
        
        # Thread
        self.thread = None
        
        

    # START AND STOP
    #############################################################################  

    def startButtonClicked(self):
        
        """ This function is called when the start / stop button is pressed.
        Do the expected action """
        
        if self.isStarted() is False : 
            self.start()
        else :
            self.stop()
            
            

    def isStarted(self):
        
        """ Returns True or False whether the scan is currently running or not """
        
        return self.thread is not None



    def start(self) :
        
        """ This function start a scan """
        
        # Test if current config is valid to start a scan
        test = False
        try : 
            config = self.gui.configManager.getConfig()
            assert config['parameter']['element'] is not None, "Parameter no set"
            assert len(config['recipe']) > 0, "Recipe is empty"
            test = True
        except Exception as e :
            self.gui.statusBar.showMessage(f'ERROR The scan cannot start with the current configuration : {str(e)}',10000)
            
        if test is True :
            
            # Prepare a new dataset in the datacenter
            self.gui.dataManager.newDataset(config)
            
            # Start a new thread
            ## Opening
            self.thread = ScanThread(self.gui.dataManager.queue, config)
            ## Signal connections
            self.thread.errorSignal.connect(self.error)
            self.thread.startStepSignal.connect(lambda stepName:self.gui.recipeManager.setStepProcessingState(stepName,'started'))
            self.thread.finishStepSignal.connect(lambda stepName:self.gui.recipeManager.setStepProcessingState(stepName,'finished'))
            self.thread.startParameterSignal.connect(lambda:self.gui.parameterManager.setProcessingState('started'))
            self.thread.finishParameterSignal.connect(lambda:self.gui.parameterManager.setProcessingState('finished'))
            self.thread.recipeCompletedSignal.connect(self.gui.recipeManager.resetStepsProcessingState)
            self.thread.finished.connect(self.finished)
            # Starting
            self.thread.start()
            
            # Start data center timer
            self.gui.dataManager.timer.start()
            
            # Update gui           
            self.gui.start_pushButton.setText('Stop')
            self.gui.pause_pushButton.setEnabled(True)
            self.gui.clear_pushButton.setEnabled(False)
            self.gui.progressBar.setValue(0)
            self.gui.configManager.importAction.setEnabled(False)
            self.gui.statusBar.showMessage(f'Scan started !',5000)
            


    def stop(self):
        
        """ This function stop manually the scan """
        
        self.disableContinuousMode()   
        self.thread.stopFlag.set()
        self.resume()
        self.thread.wait()
        
        
        
        
    # SIGNALS
    #############################################################################  
        
    def finished(self):
        
        """ This function is called when the scan thread is finished. It restores
        the GUI in a ready mode, and start a new scan if in continuous mode """
        
        self.gui.start_pushButton.setText('Start')
        self.gui.pause_pushButton.setEnabled(False)
        self.gui.clear_pushButton.setEnabled(True)
        self.gui.parameterManager.setProcessingState('idle')
        self.gui.configManager.importAction.setEnabled(True)
        self.gui.dataManager.timer.stop()
        self.gui.dataManager.sync() # once again to be sure we grabbed every data
        self.thread = None
        
        if self.isContinuousModeEnabled() : 
            self.start()
        
        
        
    def error(self,error):
        
        """ This function is called if an error occured during the scan. 
        It displays it in the status bar """
        
        self.gui.statusBar.showMessage(f'Error : {error} ',10000)
        
        
        
        
        
    # CONTINUOUS MODE
    #############################################################################
    
    def isContinuousModeEnabled(self):
        
        """ Returns True or False whether the continuous scan mode is enabled or not """
        
        return self.gui.continuous_checkBox.isChecked()
    
    
    
    def disableContinuousMode(self):
        
        """ This function disables the continuous scan mode """
        
        self.gui.continuous_checkBox.setChecked(False)
        
        
        
        
    
    # PAUSE
    #############################################################################
    
    def pauseButtonClicked(self):
        
        """ This function is called when the Pause / Resume button is clicked.
        It does the required action """
        
        if self.isStarted() :
            if self.isPaused() is False : 
                self.pause()
            else :
                self.resume()
                
                
    
    def isPaused(self):
        
        """ Returns True or False whether the scan is paused or not """
        
        return self.thread is not None and self.thread.pauseFlag.is_set() is True 
    
    
               
    def pause(self):
        
        """ This function pauses the scan """
        
        self.thread.pauseFlag.set()
        self.gui.dataManager.timer.stop()
        self.gui.pause_pushButton.setText('Resume')
        
        
        
    def resume(self):
        
        """ This function resumes the scan """
        
        self.thread.pauseFlag.clear()
        self.gui.dataManager.timer.start()
        self.gui.pause_pushButton.setText('Pause')
        
        

        
        
        
        
        
class ScanThread(QtCore.QThread):
    
    """ This thread class is dedicated to read the variable, and send its data to GUI through a queue """
    
    # Signals
    errorSignal = QtCore.pyqtSignal(object) 
    startParameterSignal = QtCore.pyqtSignal()
    finishParameterSignal = QtCore.pyqtSignal()
    startStepSignal = QtCore.pyqtSignal(object)
    finishStepSignal = QtCore.pyqtSignal(object)
    recipeCompletedSignal = QtCore.pyqtSignal()



    def __init__(self, queue, config):
        
        QtCore.QThread.__init__(self)
        self.config = config
        self.queue = queue
        
        self.pauseFlag = threading.Event()
        self.stopFlag = threading.Event()
        
        
        
    def run(self):
        
        # Load scan configuration
        parameter = self.config['parameter']['element']
        name = self.config['parameter']['name']
        startValue,endValue = self.config['range']
        nbpts = self.config['nbpts']
        logScale = self.config['log']
        
        # Creates the array of values for the parameter
        if logScale is False : 
            paramValues = np.linspace(startValue,endValue,nbpts,endpoint=True)
        else : 
            paramValues = np.logspace(m.log10(startValue),m.log10(endValue),nbpts,endpoint=True)
            
        # Start the scan
        for paramValue in paramValues : 
                        
            if self.stopFlag.is_set() is False :
                
                try : 
                    
                    # Set the parameter value
                    self.startParameterSignal.emit()
                    parameter(paramValue)
                    self.finishParameterSignal.emit()
                    
                    # Start the recipe
                    dataPoint = collections.OrderedDict()
                    dataPoint[name] = paramValue
                    dataPoint = self.processRecipe(dataPoint)
                    
                    # Send the whole data in the queue
                    if self.stopFlag.is_set() is False : self.queue.put(dataPoint)


                except Exception as e :

                    # If an error occurs, stop the scan and send an error signal
                    self.errorSignal.emit(e)
                    self.stopFlag.set()
                    
                    
                # Wait until the scan is no more in pause
                while self.pauseFlag.is_set() :
                    time.sleep(0.1)
                        
            else : 
                break
            
            
            
    def processRecipe(self,dataPoint):
        
        """ This function executes the scan recipe """

        for stepInfos in self.config['recipe'] : 
            
            if self.stopFlag.is_set() is False :
                
                # Process the recipe step
                result = self.processElement(stepInfos)
                if result is not None :
                    dataPoint[stepInfos['name']] =  result
                    
                # Wait until the scan is no more in pause
                while self.pauseFlag.is_set() :
                    time.sleep(0.1)
            
            else : 
                break
            
        self.recipeCompletedSignal.emit()
            
        return dataPoint
                 
    
        
    def processElement(self,stepInfos):
        
        """ This function processes the recipe step """
        
        element = stepInfos['element']
        stepType = stepInfos['stepType']
        
        self.startStepSignal.emit(stepInfos['name'])
            
        result = None
        if stepType == 'measure' :
            result = element()
        if stepType == 'set' :
            element(stepInfos['value'])
        if stepType == 'action' :
            if stepInfos['value'] is not None :
                element(stepInfos['value'])
            else :
                element()
            
        self.finishStepSignal.emit(stepInfos['name'])
        
        return result
    
    
    